/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/
@keyframes jst-fadein {
    from {
        background-color: transparent;
    }
    to {
        background-color: var(--c-fade);
    }
}
@keyframes jst-fadeout {
    from {
        background-color: var(--c-fade);
    }
    to {
        background-color: transparent;
    }
}
@keyframes jst-spin {
    from {
        transform: rotate(0turn);
    }
    to {
        transform: rotate(-1turn);
    }
}

.jst-fade {
    position: fixed;
    left: 0px;
    top: 0px;
    width: 100vw;
    height: 100vh;
    z-index: 10;
}
.jst-message {
    position: fixed;
    top: 50vh;
    left: 50vw;
    transform: translate(-50%, -50%);
    font-size: large;
    z-index: 11;
}
.jst-message.error {
    color: var(--c-error);
}
.jst-message.success {
    color: var(--c-success);
}
.jst-spinner {
    position: fixed;
    top: 50vh;
    left: 50vw;
    translate: -50% -50%;
    color: var(--c-warn);
    font-size: large;
    z-index: 11;
    animation-name: jst-spin;
    animation-duration: 1s;
    animation-timing-function: linear;
    animation-iteration-count: infinite;
}
nav {
    z-index: 11;
}
