/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/

use crate::backends::CacheStorage;
use anyhow::Result;

pub struct Dummy;
impl CacheStorage for Dummy {
    fn store(&self, _key: String, _value: &[u8]) -> Result<()> {
        Ok(())
    }
    fn read(&self, _key: &str) -> Result<Option<Vec<u8>>> {
        Ok(None) // sorry forgot
    }
}
