/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/

pub mod demuxers;
pub mod magic;
pub mod muxers;

pub use winter_matroska as matroska;

#[derive(Debug, Clone, Copy, PartialEq)]
pub enum ContainerFormat {
    Matroska,
    Webm,
    Flac,
    Mpeg4,
}
