/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/
nav {
    user-select: none;
    z-index: 90;
    position: fixed;
    top: 0px;
    left: 0px;
    padding: 1em;
    width: calc(100vw - 2em);
    height: 2em;
    backdrop-filter: blur(6px);
    background-color: var(--c-nav);

    display: flex;
    align-items: center;
}

nav a .logo {
    height: 1.5em;
}

nav a {
    display: inline-block;
    border: 0px solid transparent;
    border-radius: 5px;
    padding: 0.5em;
    text-decoration: none;
    color: var(--font);
    background-image: linear-gradient(transparent, transparent),
        linear-gradient(var(--accent-light), var(--accent-light));
    background-size: 100% 2px, 0 2px;
    background-position: 100% 100%, 0 100%;
    background-repeat: no-repeat;
    transition: background-size 0.15s linear;
    cursor: pointer;
}

nav a:hover {
    background-color: var(--c-nav-hover);
    background-size: 0 2px, 100% 2px;
}

nav h1 {
    margin: 0px;
    margin-left: 0.5em;
    margin-right: 0.5em;
    font-size: 1.5em;
    display: inline;
}
nav .account {
    margin-left: auto;
}
nav .account .username {
    color: var(--accent-light);
    font-weight: bold;
    margin-right: 1em;
}

nav .admin::before {
    content: "admin_panel_settings";
}

nav .settings::before {
    content: "settings";
}

nav .logout::before {
    content: "logout";
}

nav .login::before {
    content: "login";
}

nav .back::before {
    content: "arrow_back";
}

.hybrid_button p {
    display: inline;
}

@media (max-width: 1000px) {
    .hybrid_button p {
        display: none;
    }
    *::before {
        margin-right: 0px;
    }
}
@media (max-width: 750px) {
    nav .account span {
        display: none;
    }
}
@media (max-width: 500px) {
    nav .library {
        display: none;
    }
}
