/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/

pub mod import;
pub mod log;
pub mod user;

use crate::{
    FlashM, Page,
    locale::{Language, tr, trs},
    scaffold::FlashDisplay,
};
use jellycommon::routes::{
    u_admin_import, u_admin_invite_create, u_admin_invite_remove, u_admin_log,
    u_admin_update_search, u_admin_users,
};

impl Page for AdminDashboardPage<'_> {
    fn title(&self) -> String {
        "Admin Dashboard".to_string()
    }
    fn to_render(&self) -> markup::DynRender<'_> {
        markup::new!(@self)
    }
}

markup::define!(
    AdminDashboardPage<'a>(lang: &'a Language, busy: Option<&'static str>, flash: &'a FlashM, invites: &'a [String]) {
        h1 { @trs(lang, "admin.dashboard.title") }
        @FlashDisplay { flash }
        ul {
            li{a[href=u_admin_log(true)] { @trs(lang, "admin.log.warnonly") }}
            li{a[href=u_admin_log(false)] { @trs(lang, "admin.log.full") }}
        }

        a[href=u_admin_import()] { h2 { @trs(lang, "admin.import.title") }}
        @if let Some(text) = busy {
            section.message { p.warn { @text } }
        }
        form[method="POST", action=u_admin_update_search()] {
            input[type="submit", value=tr(**lang, "admin.dashboard.update_search").to_string()];
        }
        h2 { @trs(lang, "admin.dashboard.users") }
        p { a[href=u_admin_users()] { @trs(lang, "admin.dashboard.manage_users") } }
        h2 { @trs(lang, "admin.dashboard.invites") }
        form[method="POST", action=u_admin_invite_create()] {
            input[type="submit", value=tr(**lang, "admin.dashboard.create_invite").to_string()];
        }
        ul { @for t in *invites {
            li {
                form[method="POST", action=u_admin_invite_remove()] {
                    span { @t }
                    input[type="text", name="invite", value=&t, hidden];
                    input[type="submit", value=tr(**lang, "admin.dashboard.create_invite").to_string()];
                }
            }
        }}

        // h2 { "Database" }
        // @match db_stats(&database) {
        //     Ok(s) => { @s }
        //     Err(e) => { pre.error { @format!("{e:?}") } }
        // }
    }
);
