/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
    Copyright (C) 2023 tpart
*/
input {
    background-color: var(--background-dark);
    outline: none;
    box-sizing: border-box;
    height: 2.5em;
}
input[type="text"],
input[type="password"] {
    border-radius: 7px;
    padding: 0.3em;
    margin-top: 0.3em;
    border: 2px solid var(--accent-light);
}
input[type="text"]:focus,
input[type="password"]:focus {
    background-color: var(--background-light);
}
input[type="text"]:disabled,
input[type="password"]:disabled {
    border: 2px solid var(--background-disable);
}
option {
    font-family: "Cantarell", sans-serif;
}

fieldset {
    background-color: var(--background-light);
    border-radius: 8px;
}

input[type="submit"],
.play,
button {
    color: var(--font-highlight);
    padding: 0.5em;
    margin: 0.5em;
    justify-self: center;
    border: 0px solid transparent;
    background-color: var(--accent-dark);
    border-radius: 8px;
    cursor: pointer;
}
input[type="submit"]:disabled,
.play,
button:disabled {
    background-color: var(--background-disable);
}
input[type="submit"]:hover,
.play,
button:hover {
    filter: brightness(150%);
}

form.account {
    padding: 3em;
    border-radius: 1em;
    background-color: var(--background-light);

    min-width: 25em;
    position: absolute;
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);
}
form.account input {
    width: 100%;
    font-size: large;
}
form.account input,
form.account label {
    display: block;
}
form.account input[type="submit"] {
    margin: 0;
    margin-top: 1em;
    margin-bottom: 1.5em;
    font-weight: bold;
}
form.account h1 {
    margin-top: 0px;
}
form.account p {
    color: var(--font-dark);
}

legend {
    font-size: 1.5em;
}
input[type="radio"] {
    appearance: none;
    display: inline-block;
    width: 16px;
    height: 16px;
    border-radius: 8px;
    padding: 2px;
    background-clip: content-box;
    border: 2px solid var(--font);
    background-color: transparent;
    transition: background-color 0.3s;
}
input[type="radio"]:checked {
    background-color: var(--accent-light);
}

input[type="checkbox"] {
    appearance: none;
    display: inline-block;
    width: 16px;
    height: 16px;
    border-radius: 3px;
    padding: 2px;
    background-clip: content-box;
    border: 2px solid var(--font);
    background-color: transparent;
    transition: background-color 0.3s;
}
input[type="checkbox"]:checked {
    background-color: var(--accent-light);
}

fieldset label {
    transition: color 0.2s;
}
fieldset label:hover {
    color: var(--accent-light);
}

fieldset .categories {
    display: flex;
    flex-direction: row;
    flex-wrap: wrap;
}
fieldset .categories .category {
    margin-right: 2em;
    margin-left: 2em;
    min-width: max-content;
    flex-basis: auto;
    display: inline-flex;
    flex-direction: column;
}
fieldset .categories .category h3 {
    margin: 0px;
}

input.danger {
    background-color: var(--c-danger);
}
