/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/
/// <reference lib="dom" />
import "./player/mod.ts"
import "./transition.ts"
import "./backbutton.ts"
import "./dangerbutton.ts"
import "./log_live.ts"
import "./import_live.ts"
