/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/
use markup::{Render, RenderAttributeValue};
use std::{borrow::Cow, collections::HashMap, sync::LazyLock};

#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
pub enum Language {
    English,
    German,
}

static LANG_TABLES: LazyLock<HashMap<Language, HashMap<&'static str, &'static str>>> =
    LazyLock::new(|| {
        let mut k = HashMap::new();
        for (lang, source) in [
            (Language::English, include_str!("../../locale/en.ini")),
            (Language::German, include_str!("../../locale/de.ini")),
        ] {
            // TODO fallback to english
            let tr_map = source
                .lines()
                .filter_map(|line| {
                    let (key, value) = line.split_once("=")?;
                    Some((key.trim(), value.trim()))
                })
                .collect::<HashMap<&'static str, &'static str>>();
            k.insert(lang, tr_map);
        }
        k
    });

pub fn tr(lang: Language, key: &str) -> Cow<'static, str> {
    let tr_map = LANG_TABLES.get(&lang).unwrap();
    match tr_map.get(key) {
        Some(value) => Cow::Borrowed(value),
        None => Cow::Owned(format!("TR[{key}]")),
    }
}

pub fn get_translation_table(lang: &Language) -> &'static HashMap<&'static str, &'static str> {
    LANG_TABLES.get(lang).unwrap()
}

pub struct TrString<'a>(Cow<'a, str>);
impl Render for TrString<'_> {
    fn render(&self, writer: &mut impl std::fmt::Write) -> std::fmt::Result {
        self.0.render(writer)
    }
}
impl RenderAttributeValue for TrString<'_> {
    fn is_none(&self) -> bool {
        false
    }
    fn is_true(&self) -> bool {
        false
    }
    fn is_false(&self) -> bool {
        false
    }
}

pub fn escape(str: &str) -> String {
    let mut o = String::with_capacity(str.len());
    let mut last = 0;
    for (index, byte) in str.bytes().enumerate() {
        if let Some(esc) = match byte {
            b'<' => Some("&lt;"),
            b'>' => Some("&gt;"),
            b'&' => Some("&amp;"),
            b'"' => Some("&quot;"),
            _ => None,
        } {
            o += &str[last..index];
            o += esc;
            last = index + 1;
        }
    }
    o += &str[last..];
    o
}

pub fn trs<'a>(lang: &Language, key: &str) -> TrString<'a> {
    TrString(tr(*lang, key))
}
