/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/
pub mod backends;
pub mod sort;
pub mod prefix_iterator;
pub mod table;
pub mod query;
pub mod filter;

pub type Pad32 = u32;
