/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/

use crate::{Object, Tag, TypedTag};
use std::marker::PhantomData;

#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
pub struct Path(pub Vec<Tag>);

impl Path {
    pub fn get_matching_value<'a>(&self, ob: Object<'a>) -> Option<&'a [u8]> {
        fn recurse<'a>(ob: Object<'a>, path: &[Tag]) -> Option<&'a [u8]> {
            if path.len() > 1 {
                recurse(ob.get(TypedTag(path[0], PhantomData))?, path)
            } else {
                ob.get(TypedTag(path[0], PhantomData))
            }
        }
        recurse(ob, &self.0)
    }
    pub fn get_matching_values<'a>(&self, ob: Object<'a>) -> Vec<&'a [u8]> {
        fn recurse<'a>(ob: Object<'a>, out: &mut Vec<&'a [u8]>, path: &[Tag]) {
            if path.len() > 1 {
                for nested in ob.iter(TypedTag(path[0], PhantomData::<Object>)) {
                    recurse(nested, out, &path[1..]);
                }
            } else {
                out.extend(ob.iter(TypedTag(path[0], PhantomData::<&[u8]>)));
            }
        }
        let mut out = Vec::new();
        recurse(ob, &mut out, &self.0);
        out
    }
}
