/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/

.props p {
    margin: 0.4em;
    font-size: small;
    font-weight: bolder;
    display: inline-block;
    padding: 0.2em;
    background: var(--background-prop);
    border-radius: 4px;
}
.props p.federation {
    background: rgba(22, 101, 133, 0.603);
}
.props p.federation::before {
    content: "link";
}
.props p.watched {
    background: rgba(75, 175, 44, 0.644);
}
.props p.watched::before {
    content: "check";
}
.props p.progress {
    background: rgba(156, 89, 35, 0.63);
}
.props p.progress::before {
    content: "pending";
}
.props p.pending {
    background: rgba(156, 35, 69, 0.63);
}
.props p.pending::before {
    content: "playlist_add_check";
}
.props p.rating {
    background: rgba(138, 156, 35, 0.63);
}
.props p.rating::before {
    content: "stars";
}
.props p.visibility {
    background-color: rgba(56, 70, 71, 0.63);
}
.props p.visibility::before {
    content: "visibility_off";
}
/* .props p.likes::before {
    content: "thumb_up";
} */

@media (max-width: 500px) {
    .props {
        display: none;
    }
}