/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/

use crate::{Object, Registry, types::*};
use std::fmt::Debug;

pub struct ObjectInpector<'a>(pub &'a Registry, pub Object<'a>);
impl Debug for ObjectInpector<'_> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let mut s = f.debug_struct("Object");
        let mut nonexhaustive = false;
        for (i, k) in self.1.keys().enumerate() {
            let Some(info) = self.0.info(k) else {
                nonexhaustive = true;
                continue;
            };
            let Some(ty) = info.r#type else {
                nonexhaustive = true;
                continue;
            };
            match ty {
                x if x == STR => s.field(info.name, &self.1.get_typed::<&str>(i).unwrap()),
                x if x == OBJECT => s.field(info.name, &self.1.get_typed::<Object>(i).unwrap()),
                x if x == U32 => s.field(info.name, &self.1.get_typed::<u32>(i).unwrap()),
                x if x == U64 => s.field(info.name, &self.1.get_typed::<u64>(i).unwrap()),
                _ => &mut s,
            };
        }
        if nonexhaustive {
            s.finish_non_exhaustive()
        } else {
            s.finish()
        }
    }
}
