/*
    This file is part of jellything (https://codeberg.org/metamuffin/jellything)
    which is licensed under the GNU Affero General Public License (version 3); see /COPYING.
    Copyright (C) 2026 metamuffin <metamuffin.org>
*/

use crate::Page;
use jellycommon::routes::u_account_login;

impl Page for ErrorPage {
    fn title(&self) -> String {
        "Error".to_string()
    }
    fn to_render(&self) -> markup::DynRender<'_> {
        markup::new!(@self)
    }
}

markup::define! {
    ErrorPage(status: String) {
        h2 { "Error" }
        p { @status }
        // @if status == Status::NotFound {
            p { "You might need to " a[href=u_account_login()] { "log in" } ", to see this page" }
        // }
    }
}
